#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================
"""
This module exposes a function called `print_obj` that can be used to help with debugging raw java objects. It will print information about the object in question on both the Python side and on the Java server side. There's a `verbose` flag that will give you more information about the object in question.
"""
from pygw.config import java_gateway
from pygw.base import GeoWaveObject

def print_obj(to_print, verbose=False):
    """
    Print method to help with debugging.
    """
    if isinstance(to_print, GeoWaveObject):
        to_print = to_print._java_ref
    print(java_gateway.entry_point.getDebug().printObject(to_print, verbose))



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from pygw.config import geowave_pkg
from pygw.store import DataStoreOptions

class KuduOptions(DataStoreOptions):
    """
    Kudu data store options.
    """

    def __init__(self):
        super().__init__(geowave_pkg.datastore.kudu.config.KuduRequiredOptions())

    def set_kudu_master(self, kudu_master):
        """
        Sets the URL for the Kudu master node.

        Args:
            kudu_master (str): The URL for the Kudu master node.
        """
        self._java_ref.setKuduMaster(kudu_master)

    def get_kudu_master(self):
        """
        Returns:
            The URL for the Kudu master node.
        """
        return self._java_ref.getKuduMaster()



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from pygw.config import java_pkg
from pygw.config import reflection_util
from pygw.base import GeoWaveObject
from pygw.base.type_conversions import *

from enum import Enum

_binding_map = {}

class AttributeDescriptor(GeoWaveObject):
    """
    Describes a single attribute of a feature type.
    """

    def __init__(self, attribute_type, is_nilable, descriptor, j_attribute=None):
        if not isinstance(attribute_type, AttributeDescriptor.Type):
            raise AttributeDescriptor.UnknownTypeError("Invalid argument to `attribute_type`. Must be one of defined types in AttributeDescriptor.Type")
        self.field = attribute_type.value()
        self.is_nilable = is_nilable
        self.descriptor = descriptor
        if j_attribute is None:
            j_builder = java_pkg.org.geotools.feature.AttributeTypeBuilder()
            j_type_cls = reflection_util.classForName(self.field.binding)
            j_builder.binding(j_type_cls)
            j_builder.nillable(is_nilable)
            j_attribute = j_builder.buildDescriptor(descriptor)
        super().__init__(j_attribute)

    def to_java(self, value):
        """
        Converts a Python variable into its Java counterpart.

        Args:
            value (any): The Python variable to convert.
        Retuurns:
            The Java equivalent of the Python variable.
        """
        return self.field.to_java(value)

    def from_java(self, value):
        """
        Converts a Java variable into its Python counterpart.

        Args:
            value (any): The Java variable to convert.
        Retuurns:
            The Python equivalent of the Java variable.
        """
        return self.field.from_java(value)

    @classmethod
    def from_java_attribute_descriptor(cls, java_attribute_descriptor):
        """
        Constructs an attribute descriptor from a Java geotools attribute descriptor.

        Args:
            java_attribute_descriptor (AttributeDescriptor): The Java attribute descriptor.
        Retuurns:
            A `pygw.geotools.attribute_descriptor.AttributeDescriptor` that matches the Java one.
        """
        nilable = java_attribute_descriptor.isNillable()
        descriptor = java_attribute_descriptor.getName().getLocalPart()
        binding = java_attribute_descriptor.getType().getBinding().getName()
        return cls(_binding_map[binding], nilable, descriptor, java_attribute_descriptor)

    @classmethod
    def big_decimal(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for BigDecimal values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A BigDecimal `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.BIG_DECIMAL, is_nilable, descriptor)

    @classmethod
    def big_decimal_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for BigDecimal[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A BigDecimal[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.BIG_DECIMAL_ARRAY, is_nilable, descriptor)

    @classmethod
    def big_integer(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for BigInteger values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A BigInteger `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.BIG_INTEGER, is_nilable, descriptor)

    @classmethod
    def big_integer_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for BigInteger[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A BigInteger[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.BIG_INTEGER_ARRAY, is_nilable, descriptor)

    @classmethod
    def boolean(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Boolean values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Boolean `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.BOOLEAN, is_nilable, descriptor)

    @classmethod
    def boolean_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Boolean[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Boolean[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.BOOLEAN_ARRAY, is_nilable, descriptor)

    @classmethod
    def float(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Float values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Float `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.FLOAT, is_nilable, descriptor)

    @classmethod
    def float_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Float[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Float[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.FLOAT_ARRAY, is_nilable, descriptor)

    @classmethod
    def double(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Double values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Double `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.DOUBLE, is_nilable, descriptor)

    @classmethod
    def double_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Double values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Double[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.DOUBLE_ARRAY, is_nilable, descriptor)

    @classmethod
    def byte(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Byte values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Byte `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.BYTE, is_nilable, descriptor)

    @classmethod
    def byte_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Byte[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Byte[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.BYTE_ARRAY, is_nilable, descriptor)

    @classmethod
    def short(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Short values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Short `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.SHORT, is_nilable, descriptor)

    @classmethod
    def short_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Short[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Short[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.SHORT_ARRAY, is_nilable, descriptor)

    @classmethod
    def integer(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Integer values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Integer `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.INTEGER, is_nilable, descriptor)

    @classmethod
    def integer_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Integer[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Integer[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.INTEGER_ARRAY, is_nilable, descriptor)

    @classmethod
    def long(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Long values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Long `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.LONG, is_nilable, descriptor)

    @classmethod
    def long_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Long[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Long[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.LONG_ARRAY, is_nilable, descriptor)

    @classmethod
    def primitive_boolean_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for boolean[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A boolean[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.PRIMITIVE_BOOLEAN_ARRAY, is_nilable, descriptor)

    @classmethod
    def primitive_float_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for float[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A float[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.PRIMITIVE_FLOAT_ARRAY, is_nilable, descriptor)

    @classmethod
    def primitive_double_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for double[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A double[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.PRIMITIVE_DOUBLE_ARRAY, is_nilable, descriptor)

    @classmethod
    def primitive_byte_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for byte[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A byte[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.PRIMITIVE_BYTE_ARRAY, is_nilable, descriptor)

    @classmethod
    def primitive_short_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for short[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A short[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.PRIMITIVE_SHORT_ARRAY, is_nilable, descriptor)

    @classmethod
    def primitive_int_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for int[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A int[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.PRIMITIVE_INT_ARRAY, is_nilable, descriptor)

    @classmethod
    def primitive_long_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for long[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A long[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.PRIMITIVE_LONG_ARRAY, is_nilable, descriptor)

    @classmethod
    def string(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for String values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A String `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.STRING, is_nilable, descriptor)

    @classmethod
    def string_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for String[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A String[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.STRING_ARRAY, is_nilable, descriptor)

    @classmethod
    def date(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Date values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Date `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.DATE, is_nilable, descriptor)

    @classmethod
    def date_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Date[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Date[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.DATE_ARRAY, is_nilable, descriptor)

    @classmethod
    def calendar(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Calendar values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Calendar `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.CALENDAR, is_nilable, descriptor)

    @classmethod
    def calendar_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Calendar[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Calendar[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.CALENDAR_ARRAY, is_nilable, descriptor)

    @classmethod
    def point(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Point values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Point `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.POINT, is_nilable, descriptor)

    @classmethod
    def point_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Point[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Point[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.POINT_ARRAY, is_nilable, descriptor)

    @classmethod
    def multi_point(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for MultiPoint values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A MultiPoint `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.MULTI_POINT, is_nilable, descriptor)

    @classmethod
    def multi_point_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for MultiPoint[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A MultiPoint[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.MULTI_POINT_ARRAY, is_nilable, descriptor)

    @classmethod
    def line_string(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for LineString values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A LineString `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.LINE_STRING, is_nilable, descriptor)

    @classmethod
    def line_string_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for LineString[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A LineString[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.LINE_STRING_ARRAY, is_nilable, descriptor)

    @classmethod
    def multi_line_string(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for MultiLineString values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A MultiLineString `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.MULTI_LINE_STRING, is_nilable, descriptor)

    @classmethod
    def multi_line_string_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for MultiLineString[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A MultiLineString[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.MULTI_LINE_STRING_ARRAY, is_nilable, descriptor)

    @classmethod
    def polygon(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Polygon values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Polygon `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.POLYGON, is_nilable, descriptor)

    @classmethod
    def polygon_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Polygon[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Polygon[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.POLYGON_ARRAY, is_nilable, descriptor)

    @classmethod
    def multi_polygon(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for MultiPolygon values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A MultiPolygon `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.MULTI_POLYGON, is_nilable, descriptor)

    @classmethod
    def multi_polygon_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for MultiPolygon[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A MultiPolygon[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.MULTI_POLYGON_ARRAY, is_nilable, descriptor)

    @classmethod
    def geometry_collection(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for GeometryCollection values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A GeometryCollection `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.GEOMETRY_COLLECTION, is_nilable, descriptor)

    @classmethod
    def geometry_collection_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for GeometryCollection[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A GeometryCollection[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.GEOMETRY_COLLECTION_ARRAY, is_nilable, descriptor)

    @classmethod
    def geometry(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Geometry values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Geometry `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.GEOMETRY, is_nilable, descriptor)

    @classmethod
    def geometry_array(cls, descriptor, is_nilable=False):
        """
        Constructs an attribute descriptor for Geometry[] values.

        Args:
            descriptor (str): The name of the attribute.
            is_nilable (bool): Whether or not the attribute can be None. Default is False.
        Returns:
            A Geometry[] `pygw.geotools.attribute_descriptor.AttributeDescriptor`.
        """
        return cls(cls.Type.GEOMETRY_ARRAY, is_nilable, descriptor)

    class Type(Enum):
        """
        The types of attributes that are available.
        """

        BIG_DECIMAL = BigDecimalType
        BIG_DECIMAL_ARRAY = BigDecimalArrayType
        BIG_INTEGER = BigIntegerType
        BIG_INTEGER_ARRAY = BigIntegerArrayType
        BOOLEAN = BooleanType
        BOOLEAN_ARRAY = BooleanArrayType
        FLOAT = FloatType
        FLOAT_ARRAY = FloatArrayType
        DOUBLE = DoubleType
        DOUBLE_ARRAY = DoubleArrayType
        BYTE = ByteType
        BYTE_ARRAY = ByteArrayType
        SHORT = ShortType
        SHORT_ARRAY = ShortArrayType
        INTEGER = IntegerType
        INTEGER_ARRAY = IntegerArrayType
        LONG = LongType
        LONG_ARRAY = LongArrayType
        PRIMITIVE_BOOLEAN_ARRAY = PrimitiveBooleanArrayType
        PRIMITIVE_FLOAT_ARRAY = PrimitiveFloatArrayType
        PRIMITIVE_DOUBLE_ARRAY = PrimitiveDoubleArrayType
        PRIMITIVE_BYTE_ARRAY = PrimitiveByteArrayType
        PRIMITIVE_SHORT_ARRAY = PrimitiveShortArrayType
        PRIMITIVE_INT_ARRAY = PrimitiveIntArrayType
        PRIMITIVE_LONG_ARRAY = PrimitiveLongArrayType
        STRING = StringType
        STRING_ARRAY = StringArrayType
        DATE = DateType
        DATE_ARRAY = DateArrayType
        CALENDAR = CalendarType
        CALENDAR_ARRAY = CalendarArrayType
        POINT = PointType
        POINT_ARRAY = PointArrayType
        MULTI_POINT = MultiPointType
        MULTI_POINT_ARRAY = MultiPointArrayType
        LINE_STRING = LineStringType
        LINE_STRING_ARRAY = LineStringArrayType
        MULTI_LINE_STRING = MultiLineStringType
        MULTI_LINE_STRING_ARRAY = MultiLineStringArrayType
        POLYGON = PolygonType
        POLYGON_ARRAY = PolygonArrayType
        MULTI_POLYGON = MultiPolygonType
        MULTI_POLYGON_ARRAY = MultiPolygonArrayType
        GEOMETRY_COLLECTION = GeometryCollectionType
        GEOMETRY_COLLECTION_ARRAY = GeometryCollectionArrayType
        GEOMETRY = GeometryType
        GEOMETRY_ARRAY = GeometryArrayType

    class UnknownTypeError(Exception): pass

for attr in AttributeDescriptor.Type:
    _binding_map[attr.value().binding]  = attr



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

class QueryResultTransformer():
    """
    Base class for transforming GeoWave query results from Java objects to
    their Python counterparts.  All extending classes should overwrite the `transform`
    method to perform the transformation.
    """

    def transform(self, j_result):
        """
        Transforms a Java query result into a Python-friendly variant.

        Args:
            j_result (java object): The java object to transform.
        Raises:
            NotImplementedError: This is a base class and not intended to be used directly.
        Returns:
            A Python-friendly equivalent of the query result.
        """
        raise NotImplementedError



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from pygw.geotools import SimpleFeature
from pygw.geotools import SimpleFeatureType
from pygw.geotools import AttributeDescriptor

from ..query_result_transformer import QueryResultTransformer

class SimpleFeatureTransformer(QueryResultTransformer):
    """
    Transforms Java SimpleFeature query results into pgyw.geotools.SimpleFeature
    results.  In order to accomplish this, the pygw variant of the SimpleFeatureType
    has to be constructed from the feature.  In order to avoid doing this for every
    result, there is a feature type cache that the transform function can pull from.
    """
    def __init__(self):
        self._feature_type_cache = {}

    def transform(self, j_result):
        """
        Transform the given Java SimpleFeature into a pygw.geotools.SimpleFeature.

        Args:
            j_result (Java SimpleFeature): A geotools SimpleFeature Java object.
        Returns:
            A `pygw.geotools.simple_feature.SimpleFeature`.
        """
        j_sft = j_result.getFeatureType()
        type_name = j_sft.getTypeName()
        if type_name in self._feature_type_cache:
            sft = self._feature_type_cache[type_name]
        else:
            j_attrs = j_sft.getAttributeDescriptors().iterator()
            descriptors = []
            while j_attrs.hasNext():
                descriptors.append(AttributeDescriptor.from_java_attribute_descriptor(j_attrs.next()))
            sft = SimpleFeatureType(j_sft, descriptors)
            self._feature_type_cache[type_name] = sft
        return SimpleFeature(sft, j_result)



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from pygw.config import java_pkg
from pygw.base import GeoWaveObject

from .simple_feature_type import SimpleFeatureType
from .simple_feature import SimpleFeature

class SimpleFeatureBuilder(GeoWaveObject):
    """
    Builds SimpleFeature instances for a given SimpleFeatureType.
    """

    def __init__(self, feature_type):
        """
        Constructs a new simple feature builder using the given feature type.

        Args:
            feature_type (pygw.geotools.simple_feature_type.SimpleFeatureType): The feature type of the feature.
        """
        assert isinstance(feature_type, SimpleFeatureType)
        self._feature_type = feature_type
        j_builder = java_pkg.org.geotools.feature.simple.SimpleFeatureBuilder(feature_type._java_ref)
        super().__init__(j_builder)

    def set_attr(self, descriptor, value):
        """
        Sets an attribute of the feature to the given value.

        Args:
            descriptor (str): The name or index of the attribute to set.
            value (any): The value of the attribute.
        Returns:
            This feature builder.
        """
        attr = self._feature_type.get_attribute(descriptor)
        if attr is None:
            raise SimpleFeatureBuilder.NoSuchAttributeInTypeError("No matching attribute for {}".format(descriptor))
        j_value = attr.to_java(value)
        self._java_ref.set(descriptor, j_value)
        return self

    def build(self, id):
        """
        Constructs the configured feature.

        Args:
            id (str): The feature ID to use.
        Returns:
            The constructed `pygw.geotools.simple_feature.SimpleFeature`.
        """
        j_feature = self._java_ref.buildFeature(str(id))
        return SimpleFeature(self._feature_type, j_feature)

    class NoSuchAttributeInTypeError(Exception):
        """
        Error that is raised when attempting to set an attribute using a descriptor
        that does not match any of the attributes in the feature type.
        """
        pass



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from pygw.base import GeoWaveObject
from pygw.base.type_conversions import GeometryType

from .simple_feature_type import SimpleFeatureType

class SimpleFeature(GeoWaveObject):
    """
    A Simple (Vector) Feature.
    """

    def __init__(self, feature_type, java_ref):
        assert isinstance(feature_type, SimpleFeatureType)
        self._feature_type = feature_type
        super().__init__(java_ref)

    def get_id(self):
        """
        Returns:
            The ID of the feature.
        """
        return self._java_ref.getID()

    def get_type(self):
        """
        Returns:
            The `pygw.geotools.simple_feature_type.SimpleFeatureType` of the feature.
        """
        return self._feature_type

    def get_feature_type(self):
        """
        Identical to `get_type`.

        Returns:
            The `pygw.geotools.simple_feature_type.SimpleFeatureType` of the feature.
        """
        return self.get_type()

    def get_attributes(self):
        """
        Gets all of the attribute values of the feature in a single list.

        Returns:
            A list containing all attribute values of the feature.
        """
        j_values = self._java_ref.getAttributes()
        descriptors = self._feature_type.get_attribute_descriptors()
        values = []
        for i in range(len(j_values)):
            values.append(descriptors[i].from_java(j_values[i]))
        return values

    def get_attribute(self, attribute):
        """
        Gets a single attribute value by name or by index.

        Args:
            attribute (str or int): Name or index of the attribute to get.
        Returns:
            The value of the requested attribute.
        """
        j_value = self._java_ref.getAttribute(attribute)
        attr = self._feature_type.get_attribute(attribute)
        return attr.from_java(j_value)

    def get_attribute_count(self):
        """
        Returns:
            The number of attributes that the feature has.
        """
        return self._java_ref.getAttributeCount()

    def get_default_geometry(self):
        """
        Returns:
            The default geometry of the feature.
        """
        j_geom = self._java_ref.getDefaultGeometry()
        return GeometryType().from_java(j_geom)

    def to_dict(self, id_column="id"):
        """
        Convert this feature to a dictionary, including the feature ID.

        Args:
            id_column (str): The key for the feature ID. Default is "id".
        Returns:
            The attributes and feature ID of this feature in a dictionary.
        """
        j_values = self._java_ref.getAttributes()
        descriptors = self._feature_type.get_attribute_descriptors()
        feature_dict = {}
        feature_dict[id_column] = self._java_ref.getID()
        for i in range(len(j_values)):
            feature_dict[descriptors[i].descriptor] = descriptors[i].from_java(j_values[i])
        return feature_dict



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================
"""
This project aims to provide Python classes that allow users to interact with a GeoWave data store using the same workflows that are available in the programmatic Java API.

## Environment
- Python >=3,<=3.7
- A virtualenv with `requirements.txt` installed

### Installation
- Set up virtualenv: `virtualenv -p python3 venv`
- Activate virtualenv: `source venv/bin/activate`
- Install requirements: `pip install -r requirements.txt`

## Usage
In order to use `pygw`, you must have an instance of GeoWave Py4J Java Gateway Server running.  The gateway can be started by using the GeoWave command `geowave python rungateway`.

You can then import `pygw` classes into your Python environment.

## Example
The following is an example of how `pygw` might be used to write and query some feature data:
```python
from datetime import datetime

from shapely.geometry import Point

from pygw.store import DataStoreFactory
from pygw.store.rocksdb import RocksDBOptions
from pygw.geotools import SimpleFeatureBuilder
from pygw.geotools import SimpleFeatureTypeBuilder
from pygw.geotools import AttributeDescriptor
from pygw.geotools import FeatureDataAdapter
from pygw.index import SpatialIndexBuilder
from pygw.query import VectorQueryBuilder

# Create a RocksDB data store
options = RocksDBOptions()
options.set_geowave_namespace("geowave.example")
# NOTE: Directory is relative to the JVM working directory.
options.set_directory("./datastore")
datastore = DataStoreFactory.create_data_store(options)

# Create a point feature type
point_type_builder = SimpleFeatureTypeBuilder()
point_type_builder.set_name("TestPointType")
point_type_builder.add(AttributeDescriptor.point("the_geom"))
point_type_builder.add(AttributeDescriptor.date("date"))
point_type = point_type_builder.build_feature_type()

# Create a builder for this feature type
point_feature_builder = SimpleFeatureBuilder(point_type)

# Create an adapter for point type
point_type_adapter = FeatureDataAdapter(point_type)

# Create a Spatial Index
index = SpatialIndexBuilder().create_index()

# Registering the point adapter with the spatial index to your datastore
datastore.add_type(point_type_adapter, index)

# Creating a writer to ingest data
writer = datastore.create_writer(point_type_adapter.get_type_name())

# Write some features to the data store
point_feature_builder.set_attr("the_geom", Point(1, 1))
point_feature_builder.set_attr("date", datetime.now())
writer.write(point_feature_builder.build("feature1"))

point_feature_builder.set_attr("the_geom", Point(5, 5))
point_feature_builder.set_attr("date", datetime.now())
writer.write(point_feature_builder.build("feature2"))

point_feature_builder.set_attr("the_geom", Point(-5, -5))
point_feature_builder.set_attr("date", datetime.now())
writer.write(point_feature_builder.build("feature3"))

# Close the writer
writer.close()

# Query the data (with no constraints)
query = VectorQueryBuilder().build()
results = datastore.query(query)
for feature in results:
    print(feature.get_id())
    print(feature.get_default_geometry())
results.close()
```
"""
from pkg_resources import get_distribution
from pkg_resources import DistributionNotFound

try:
    version = get_distribution('pygw').version
except DistributionNotFound:
    from maven_version import get_maven_version
    version = get_maven_version();

__version__ = version



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from pygw.config import geowave_pkg
from pygw.store import DataStoreOptions

class BigTableOptions(DataStoreOptions):
    """
    BigTable data store options.
    """

    def __init__(self):
        super().__init__(geowave_pkg.datastore.bigtable.config.BigTableOptions())

    def set_scan_cache_size(self, scan_cache_size):
        """
        Sets the scan cache size of the BigTable instance.

        Args:
            scan_cache_size (int): The scan cache size to use.
        """
        self._java_ref.setScanCacheSize(scan_cache_size)

    def get_scan_cache_size(self):
        """
        Returns:
            The scan cache size of the BigTable instance.
        """
        return self._java_ref.getScanCacheSize()

    def set_project_id(self, project_id):
        """
        Sets the project id of the BigTable data store.

        Args:
            project_id (str): The project ID to use.
        """
        self._java_ref.setProjectId(project_id)

    def get_project_id(self):
        """
        Returns:
            The project ID of the data store.
        """
        return self._java_ref.getProjectId()

    def set_instance_id(self, instance_id):
        """
        Sets the instance id of the BigTable data store.

        Args:
            instance_id (str): The instance ID to use.
        """
        self._java_ref.setInstanceId(instance_id)

    def get_instance_id(self):
        """
        Returns:
            The instance ID of the data store.
        """
        return self._java_ref.getInstanceId()



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================
"""
This module contains the classes needed for querying vector data.

It contains the following import shortcuts:
```python
from pygw.query.vector import SpatialTemporalConstraintsBuilder
from pygw.query.vector import VectorQueryConstraintsFactory
from pygw.query.vector import FilterFactory
from pygw.query.vector import SimpleFeatureTransformer
from pygw.query.vector import VectorQueryBuilder
```
"""

from .spatial_temporal_constraints_builder import SpatialTemporalConstraintsBuilder
from .vector_query_constraints_factory import VectorQueryConstraintsFactory
from .filter_factory import FilterFactory
from .simple_feature_transformer import SimpleFeatureTransformer
from .vector_query_builder import VectorQueryBuilder



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from pygw.config import geowave_pkg
from pygw.config import java_pkg
from pygw.store import DataStoreOptions

class DynamoDBOptions(DataStoreOptions):
    """
    DynamoDB data store options.
    """

    def __init__(self):
        super().__init__(geowave_pkg.datastore.dynamodb.config.DynamoDBOptions())

    def set_region(self, region):
        """
        Sets the AWS region of the DynamoDB data store. For example `us-east-1`
        (specify either endpoint or region not both).

        Args:
            region (str): The AWS region to use.
        """
        if region is None:
            j_region = None
        else:
            j_region = java_pkg.com.amazonaws.regions.Regions.fromName(region)
        self._java_ref.setRegion(j_region)

    def get_region(self):
        """
        Returns:
            The AWS regiion of the DynamoDB data store.
        """
        j_region = self._java_ref.getRegion()
        if j_region is None:
            return j_region
        return j_region.getName()

    def set_endpoint(self, endpoint):
        """
        Sets the endpoint to connect to (specify either endpoint or region not both).

        Args:
            endpoint (str): The endpoint to connect to.
        """
        self._java_ref.setEndpoint(endpoint)

    def get_endpoint(self):
        """
        Returns:
            The endpoint to connect to.
        """
        return self._java_ref.getEndpoint()

    def set_write_capacity(self, write_capacity):
        """
        Sets the write capacity of the DynamoDB data store.

        Args:
            write_capacity (int): The write capacity.
        """
        self._java_ref.setWriteCapacity(write_capacity)

    def get_write_capacity(self):
        """
        Returns:
            The write capacity of the data store.
        """
        return self._java_ref.getWriteCapacity()

    def set_read_capacity(self, read_capacity):
        """
        Sets the read capacity of the DynamoDB data store.

        Args:
            read_capacity (int): The read capacity.
        """
        self._java_ref.setReadCapacity(read_capacity)

    def get_read_capacity(self):
        """
        Returns:
            The read capacity of the data store.
        """
        return self._java_ref.getReadCapacity()

    def set_enable_cache_response_metadata(self, enable_cache_response_metadata):
        """
        Sets whether or not to cache response metadata.

        Args:
            enable_cache_response_metadata (bool): Whether or not to cache response metadata.
        """
        self._java_ref.setEnableCacheResponseMetadata(enable_cache_response_metadata)

    def is_enable_cache_response_metadata(self):
        """
        Returns:
            True if response metadata will be cached, False otherwise.
        """
        return self._java_ref.isEnableCacheResponseMetadata()

    def set_protocol(self, protocol):
        """
        Sets the protocol of the connection to use. Either 'http' or 'https'.

        Args:
            protocol (str): The protocol to use.
        """
        if protocol is None:
            j_protocol = None
        else:
            j_protocol = java_pkg.com.amazonaws.Protocol.valueOf(protocol.upper())
        self._java_ref.setProtocol(j_protocol)

    def get_protocol(self):
        """
        Returns:
            The protocol of the connection to the data store.
        """
        j_protocol = self._java_ref.getProtocol()
        if j_protocol is None:
            return j_protocol
        return j_protocol.name()

    def set_max_connections(self, max_connections):
        """
        Sets the maximum number of connections to the data store.

        Args:
            max_connections (int): The maximum number of connections.
        """
        self._java_ref.setMaxConnections(max_connections)

    def get_max_connections(self):
        """
        Returns:
            The maximum number of connections.
        """
        return self._java_ref.getMaxConnections()



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================
"""
This module contains classes specific to Redis data stores.

It contains the following import shortcuts:
```python
from pygw.store.redis import RedisOptions
```
"""

from .options import RedisOptions



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from pygw.config import java_pkg
from pygw.base import GeoWaveObject

from .simple_feature_type import SimpleFeatureType
from .attribute_descriptor import AttributeDescriptor

class SimpleFeatureTypeBuilder(GeoWaveObject):
    """
    Builds `pygw.geotools.simple_feature_type.SimpleFeatureType` instances.
    """

    def __init__(self):
        self.attributes = []
        super().__init__(java_pkg.org.geotools.feature.simple.SimpleFeatureTypeBuilder())

    def set_name(self, name):
        """
        Sets the name of the feature type.

        Args:
            name (str): The name to use.
        Returns:
            This feature type builder.
        """
        self._java_ref.setName(name)
        return self

    def set_namespace_uri(self, namespace_uri):
        """
        Sets the namespace URI of the feature type.

        Args:
            namespace_uri (str): The namespace URI to use.
        Returns:
            This feature type builder.
        """
        self._java_ref.setNamespaceURI(namespace_uri)
        return self

    def set_srs(self, srs):
        """
        Sets the spatial reference system of the feature type.

        Args:
            srs (str): The spatial reference system to use.
        Returns:
            This feature type builder.
        """
        self._java_ref.setSRS(srs)
        return self

    def add(self, attribute_descriptor):
        """
        Adds an attribute to the feature type.

        Args:
            attribute_descriptor (pygw.geotools.attribute_descriptor.AttributeDescriptor): The attribute to add.
        Returns:
            This feature type builder.
        """
        if isinstance(attribute_descriptor, AttributeDescriptor):
            self.attributes.append(attribute_descriptor)
            self._java_ref.add(attribute_descriptor._java_ref)
            return self
        else:
            raise ValueError("attribute_descriptor should be of type AttributeDescriptor")

    def build_feature_type(self):
        """
        Builds the configured feature type.

        Returns:
            A `pygw.geotools.simple_feature_type.SimpleFeatureType` with the given configuration.
        """
        return SimpleFeatureType(self._java_ref.buildFeatureType(), self.attributes)



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from pygw.config import geowave_pkg

from .data_store import DataStore
from .data_store_options import DataStoreOptions

class DataStoreFactory():
    """
    Factory class for creating a data store from a given set of options.
    """

    @classmethod
    def create_data_store(cls, options):
        """
        Creates a data store from a set of options for a specific backend type.

        Args:
            options (pygw.store.data_store_options.DataStoreOptions): The options for the data store.
        Returns:
            The `pygw.store.data_store.DataStore` referenced by the given options.
        """
        assert isinstance(options, DataStoreOptions)
        j_ds = geowave_pkg.core.store.api.DataStoreFactory.createDataStore(options._java_ref)
        return DataStore(j_ds)



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from datetime import datetime
from decimal import Decimal

from py4j.java_gateway import is_instance_of
from shapely.geometry import *

from pygw.base.type_conversions import *
from pygw.config import java_gateway

def java_round_trip(type, value, check_binding=True):
    j_value = type.to_java(value)

    if value is not None and check_binding:
        assert is_instance_of(java_gateway, j_value, type.binding)

    py_value = type.from_java(j_value)
    assert value == py_value

def test_big_decimal():
    big_decimal_type = BigDecimalType()
    java_round_trip(big_decimal_type, Decimal('1.1234'))
    java_round_trip(big_decimal_type, None)

    big_decimal_array_type = BigDecimalArrayType()
    java_round_trip(big_decimal_array_type, [Decimal('1.1234'), None, Decimal('-15.821')])
    java_round_trip(big_decimal_array_type, [])
    java_round_trip(big_decimal_array_type, None)

def test_big_integer():
    big_integer_type = BigIntegerType()
    java_round_trip(big_integer_type, 1234567890123456789012345678901234567890)
    java_round_trip(big_integer_type, 123231)
    java_round_trip(big_integer_type, None)

    big_integer_array_type = BigIntegerArrayType()
    java_round_trip(big_integer_array_type, [1234567890123456789012345678901234567890, None, 123231])
    java_round_trip(big_integer_array_type, [])
    java_round_trip(big_integer_array_type, None)

def test_boolean():
    boolean_type = BooleanType()
    java_round_trip(boolean_type, True)
    java_round_trip(boolean_type, False)
    java_round_trip(boolean_type, None)

    boolean_array_type = BooleanArrayType()
    java_round_trip(boolean_array_type, [True, None, False])
    java_round_trip(boolean_array_type, [])
    java_round_trip(boolean_array_type, None)

    primitive_boolean_array_type = PrimitiveBooleanArrayType()
    java_round_trip(primitive_boolean_array_type, [True, False, False, True])
    java_round_trip(primitive_boolean_array_type, [])
    java_round_trip(primitive_boolean_array_type, None)

def test_float():
    float_type = FloatType()
    java_round_trip(float_type, 1.234, False)
    java_round_trip(float_type, -10, False)
    java_round_trip(float_type, None)

    float_array_type = FloatArrayType()
    java_round_trip(float_array_type, [1.234, -10, None])
    java_round_trip(float_array_type, [])
    java_round_trip(float_array_type, None)

    primitive_float_array_type = PrimitiveFloatArrayType()
    java_round_trip(primitive_float_array_type, [1.234, -10, 15.5])
    java_round_trip(primitive_float_array_type, [])
    java_round_trip(primitive_float_array_type, None)

def test_double():
    double_type = DoubleType()
    java_round_trip(double_type, 1.234)
    java_round_trip(double_type, -10)
    java_round_trip(double_type, None)

    double_array_type = DoubleArrayType()
    java_round_trip(double_array_type, [1.234, -10, None])
    java_round_trip(double_array_type, [])
    java_round_trip(double_array_type, None)

    primitive_double_array_type = PrimitiveDoubleArrayType()
    java_round_trip(primitive_double_array_type, [1.234, -10, 15.5])
    java_round_trip(primitive_double_array_type, [])
    java_round_trip(primitive_double_array_type, None)

def test_byte():
    byte_type = ByteType()
    java_round_trip(byte_type, 32, False)
    java_round_trip(byte_type, -127, False)
    java_round_trip(byte_type, None)

    byte_array_type = ByteArrayType()
    java_round_trip(byte_array_type, [32, None, -127])
    java_round_trip(byte_array_type, b'1234')
    java_round_trip(byte_array_type, b'')
    java_round_trip(byte_array_type, None)

    primitive_byte_array_type = PrimitiveByteArrayType()
    java_round_trip(primitive_byte_array_type, b'1234')
    java_round_trip(primitive_byte_array_type, b'')
    java_round_trip(primitive_byte_array_type, None)

def test_short():
    short_type = ShortType()
    java_round_trip(short_type, 3232, False)
    java_round_trip(short_type, -1207, False)
    java_round_trip(short_type, None)

    short_array_type = ShortArrayType()
    java_round_trip(short_array_type, [3232, None, -1207])
    java_round_trip(short_array_type, [])
    java_round_trip(short_array_type, None)

    primitive_short_array_type = PrimitiveShortArrayType()
    java_round_trip(primitive_short_array_type, [3232, -1207, 0])
    java_round_trip(primitive_short_array_type, [])
    java_round_trip(primitive_short_array_type, None)

def test_integer():
    integer_type = IntegerType()
    java_round_trip(integer_type, 3232234)
    java_round_trip(integer_type, -1207234)
    java_round_trip(integer_type, None)

    integer_array_type = IntegerArrayType()
    java_round_trip(integer_array_type, [3232234, None, -1207234])
    java_round_trip(integer_array_type, [])
    java_round_trip(integer_array_type, None)

    primitive_int_array_type = PrimitiveIntArrayType()
    java_round_trip(primitive_int_array_type, [3232234, -1207234, 0])
    java_round_trip(primitive_int_array_type, [])
    java_round_trip(primitive_int_array_type, None)

def test_long():
    long_type = LongType()
    java_round_trip(long_type, 3232234234)
    java_round_trip(long_type, -3207234234)
    java_round_trip(long_type, None)

    long_array_type = LongArrayType()
    java_round_trip(long_array_type, [3232234234, None, -3207234234])
    java_round_trip(long_array_type, [])
    java_round_trip(long_array_type, None)

    primitive_long_array_type = PrimitiveLongArrayType()
    java_round_trip(primitive_long_array_type, [3232234234, -3207234234, 0])
    java_round_trip(primitive_long_array_type, [])
    java_round_trip(primitive_long_array_type, None)

def test_string():
    string_type = StringType()
    java_round_trip(string_type, "test")
    java_round_trip(string_type, u"✓ unicode check")
    java_round_trip(string_type, "")
    java_round_trip(string_type, None)

    string_array_type = StringArrayType()
    java_round_trip(string_array_type, ["test", u"✓ unicode check", "", None])
    java_round_trip(string_array_type, [])
    java_round_trip(string_array_type, None)

def test_date():
    date_type = DateType()
    java_round_trip(date_type, datetime.fromtimestamp(1563826071))
    java_round_trip(date_type, datetime.fromtimestamp(0))
    java_round_trip(date_type, None)

    date_array_type = DateArrayType()
    java_round_trip(date_array_type, [datetime.fromtimestamp(1563826071), datetime.fromtimestamp(0), None])
    java_round_trip(date_array_type, [])
    java_round_trip(date_array_type, None)

def test_calendar():
    calendar_type = CalendarType()
    java_round_trip(calendar_type, datetime.fromtimestamp(1563826071))
    java_round_trip(calendar_type, datetime.fromtimestamp(0))
    java_round_trip(calendar_type, None)

    calendar_array_type = CalendarArrayType()
    java_round_trip(calendar_array_type, [datetime.fromtimestamp(1563826071), datetime.fromtimestamp(0), None])
    java_round_trip(calendar_array_type, [])
    java_round_trip(calendar_array_type, None)

_test_point = Point(1, 1)
_test_multi_point = MultiPoint([[0.5, 0.5], [1, 1]])
_test_line_string = LineString([[0.5, 0.5], [1, 1]])
_test_multi_line_string = MultiLineString([[[0.5, 0.5], [1, 1]], [[-0.5, -0.5], [1, 1]]])
_test_polygon = Polygon([[0, 0], [0, 1], [1, 1], [1, 0], [0, 0]])
_test_polygon2 = Polygon([[0, 0], [0, -1], [-1, -1], [-1, 0], [0, 0]])
_test_multi_polygon = MultiPolygon([_test_polygon, _test_polygon2])

def test_point():
    point_type = PointType()
    java_round_trip(point_type, _test_point)
    java_round_trip(point_type, None)

    point_array_type = PointArrayType()
    java_round_trip(point_array_type, [_test_point, None])
    java_round_trip(point_array_type, [])
    java_round_trip(point_array_type, None)

def test_multi_point():
    multi_point_type = MultiPointType()
    java_round_trip(multi_point_type, _test_multi_point)
    java_round_trip(multi_point_type, None)

    multi_point_array_type = MultiPointArrayType()
    java_round_trip(multi_point_array_type, [_test_multi_point, None])
    java_round_trip(multi_point_array_type, [])
    java_round_trip(multi_point_array_type, None)

def test_line_string():
    line_string_type = LineStringType()
    java_round_trip(line_string_type, _test_line_string)
    java_round_trip(line_string_type, None)

    line_string_array_type = LineStringArrayType()
    java_round_trip(line_string_array_type, [_test_line_string, None])
    java_round_trip(line_string_array_type, [])
    java_round_trip(line_string_array_type, None)

def test_multi_line_string():
    multi_line_string_type = MultiLineStringType()
    java_round_trip(multi_line_string_type, _test_multi_line_string)
    java_round_trip(multi_line_string_type, None)

    multi_line_string_array_type = MultiLineStringArrayType()
    java_round_trip(multi_line_string_array_type, [_test_multi_line_string, None])
    java_round_trip(multi_line_string_array_type, [])
    java_round_trip(multi_line_string_array_type, None)

def test_polygon():
    polygon_type = PolygonType()
    java_round_trip(polygon_type, _test_polygon)
    java_round_trip(polygon_type, None)

    polygon_array_type = PolygonArrayType()
    java_round_trip(polygon_array_type, [_test_polygon, None])
    java_round_trip(polygon_array_type, [])
    java_round_trip(polygon_array_type, None)

def test_multi_polygon():
    multi_polygon_type = MultiPolygonType()
    java_round_trip(multi_polygon_type, _test_multi_polygon)
    java_round_trip(multi_polygon_type, None)

    multi_polygon_array_type = MultiPolygonArrayType()
    java_round_trip(multi_polygon_array_type, [_test_multi_polygon, None])
    java_round_trip(multi_polygon_array_type, [])
    java_round_trip(multi_polygon_array_type, None)

def test_geometry():
    geometry_type = GeometryType()
    java_round_trip(geometry_type, _test_point)
    java_round_trip(geometry_type, _test_multi_point)
    java_round_trip(geometry_type, _test_line_string)
    java_round_trip(geometry_type, _test_multi_line_string)
    java_round_trip(geometry_type, _test_polygon)
    java_round_trip(geometry_type, _test_multi_polygon)
    java_round_trip(geometry_type, None)

    geometry_array_type = GeometryArrayType()
    java_round_trip(geometry_array_type, [_test_point, _test_multi_point, _test_line_string, _test_multi_line_string, _test_polygon, _test_multi_polygon, None])
    java_round_trip(geometry_array_type, [])
    java_round_trip(geometry_array_type, None)

def test_geometry_collection():
    geometry_collection_type = GeometryCollectionType()
    test_geometry_collection = GeometryCollection([_test_point, _test_multi_point, _test_line_string, _test_multi_line_string, _test_polygon, _test_multi_polygon])
    java_round_trip(geometry_collection_type, test_geometry_collection)
    java_round_trip(geometry_collection_type, None)

    geometry_collection_array_type = GeometryCollectionArrayType()
    java_round_trip(geometry_collection_array_type, [test_geometry_collection, None])
    java_round_trip(geometry_collection_array_type, [])
    java_round_trip(geometry_collection_array_type, None)



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from pygw.config import geowave_pkg

from ..query_builder import QueryBuilder

class VectorStatisticsQueryBuilder(QueryBuilder):
    """
    A builder for creating statistics queries for vector data.
    """

    def __init__(self):
        j_stats_qbuilder = geowave_pkg.core.geotime.store.query.api.VectorStatisticsQueryBuilder.newBuilder()
        super().__init__(j_stats_qbuilder)

    def bbox(self):
        # TODO: support FeatureBoundingBoxStatistics builder
        return self._java_ref.bbox()

    def time_range(self):
        # TODO: support FeatureTimeRangeStatistics builder
        return self._java_ref.timeRange()



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from pygw.base import GeoWaveObject
from pygw.base import CloseableIterator
from pygw.base import Writer
from pygw.base import DataTypeAdapter
from pygw.config import java_gateway
from pygw.config import geowave_pkg
from pygw.query import Query
from pygw.index import Index

class DataStore(GeoWaveObject):
    """
    This class models the DataStore interface methods.
    """

    def __init__(self, java_ref):
        super().__init__(java_ref)

    def ingest(self, url, *indices, ingest_options=None):
        """
        Ingest from URL.

        If this is a directory, this method will recursively search for valid files to
        ingest in the directory. This will iterate through registered IngestFormatPlugins to find one
        that works for a given file.

        Args:
            url (str): The URL for data to read and ingest into this data store.
            *indices (pygw.index.index.Index): Index to ingest into.
            ingest_options: Options for ingest (Not yet supported).
        """
        # TODO: Ingest Options
        if ingest_options: raise NotImplementedError()

        assert isinstance(url, str)

        n = len(indices)
        j_index_class = geowave_pkg.core.store.api.Index
        j_index_arr = java_gateway.new_array(j_index_class, n)
        for idx, name in enumerate(indices):
                j_index_arr[idx] = name._java_ref
        java_url = java_gateway.jvm.java.net.URL(url)
        self._java_ref.ingest(java_url, ingest_options, j_index_arr)

    def query(self, q):
        """
        Returns all data in this data store that matches the query parameter. All data that matches the
        query will be returned as an instance of the native data type. The Iterator must be closed when
        it is no longer needed - this wraps the underlying scanner implementation and closes underlying
        resources.

        Args:
            q (pygw.query.query.Query): The query to preform.
        Returns:
            A closeable iterable of results.  The `pygw.base.closeable_iterator.CloseableIterator.close` method should be called
            on the iterator when it is done being used.
        """
        assert isinstance(q, Query)
        j_query = q._java_ref
        return iter(CloseableIterator(self._java_ref.query(j_query), q.result_transformer))

    def aggregate(self, q):
        raise NotImplementedError

    def get_types(self):
        """
        Get all the data type adapters that have been used within this data store.

        Returns:
            List of `pygw.base.data_type_adapter.DataTypeAdapter` used in the data store.
        """
        j_adapter_arr = self._java_ref.getTypes()
        return [DataTypeAdapter(j_adpt) for j_adpt in j_adapter_arr]

    def query_statistics(self, q):
        raise NotImplementedError

    def aggregate_statistics(self, q):
        raise NotImplementedError

    def get_indices(self, type_name=None):
        """
        Get the indices that have been registered with this data store for a given type.

        Gets all registered indices if `type_name` is None.

        Args:
            type_name (str): The name of the type.
        Returns:
            List of `pygw.index.index.Index` in the data store.
        """
        if type_name:
            j_indices = self._java_ref.getIndices(type_name)
        else:
            j_indices = self._java_ref.getIndices()
        return [Index(j_index) for j_index in j_indices]

    def copy_to(self, other, q=None):
        """
        Copy data from this data store to another.

        All data is copied if `q` is None, else only the data queried by `q`.

        Args:
            other (pygw.store.data_store.DataStore): The data store to copy to.
            q (pygw.query.query.Query): Query filter for data to be copied.
        """
        assert isinstance(other, DataStore)

        if q:
            assert isinstance(q, Query)
            q = q._java_ref

        self._java_ref.copyTo(other._java_ref, q)

    def add_index(self, type_name, *indices):
        """
        Add new indices for the given type. If there is data in other indices for this type, for
        consistency it will need to copy all of the data into the new indices, which could be a long
        process for lots of data.

        Args:
            type_name (str): Name of data type to register indices to.
            *indices (pygw.index.index.Index): Index to add.
        """
        assert isinstance(type_name,str)

        n = len(indices)
        j_index_class = geowave_pkg.core.store.api.Index
        j_index_arr = java_gateway.new_array(j_index_class,n)
        for idx, py_obj in enumerate(indices):
                j_index_arr[idx] = py_obj._java_ref

        self._java_ref.addIndex(type_name,j_index_arr)

    def remove_index(self, index_name, type_name=None):
        """
        Remove an index for a given data type.

        If `type_name` is None, the specified index is removed for all types.

        Args:
            index_name (str): Name of the index to be removed.
            type_name (str): Name of data type to remove.
        Raises:
            Exception: If the index was the last index of a type.
        """
        if type_name:
            self._java_ref.removeIndex(index_name,type_name)
        else:
            self._java_ref.removeIndex(index_name)


    def remove_type(self, type_name):
        """
        Remove all data and statistics associated with the given type.

        Args:
            type_name (str): Name of the data type.
        """
        assert isinstance(type_name, str)

        self._java_ref.removeType(type_name)


    def delete(self, q):
        """
        Delete all data in this data store that matches the query parameter.

        Args:
            q (pygw.query.query.Query): The query criteria to use for deletion.
        Returns:
            True on success, False on fail.
        """
        assert isinstance(q, Query)

        return self._java_ref.delete(q._java_ref)

    def delete_all(self):
        """
        Delete ALL data and ALL metadata for this datastore.

        Returns:
            True on success, False on fail.
        """

        return self._java_ref.deleteAll()

    def add_type(self, type_adapter, *initial_indices):
        """
        Add this type to the data store. This only needs to be called one time per type.

        Args:
            type_adapter (pygw.base.data_type_adapter.DataTypeAdapter): The data type adapter to add to the data store.
            *initial_indices (pygw.index.index.Index): The initial indices for this type.
        """
        assert isinstance(type_adapter,DataTypeAdapter)

        n = len(initial_indices)
        j_index_class = geowave_pkg.core.store.api.Index
        j_index_arr = java_gateway.new_array(j_index_class,n)
        for idx, py_obj in enumerate(initial_indices):
                j_index_arr[idx] = py_obj._java_ref

        self._java_ref.addType(type_adapter._java_ref,j_index_arr)

    def create_writer(self, type_adapter_name):
        """
        Returns an index writer to perform batched write operations for the given data type name.

        Assumes the type has already been used previously or added using `add_type` and assumes one or
        more indices have been provided for this type.

        Args:
            type_name (str): The name of the type to write to.
        Returns:
            A `pygw.base.writer.Writer`, which can be used to write entries into the data store of the given type.
        """
        j_writer = self._java_ref.createWriter(type_adapter_name)

        if j_writer is None:
            return None

        return Writer(j_writer)



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================
"""
This module contains classes that are used in constructing queries and their constraints.

It contains the following import shortcuts:
```python
from pygw.query import Query
from pygw.query import QueryConstraints
from pygw.query import QueryHintKey
from pygw.query import VectorQueryBuilder
from pygw.query import FilterFactory
```
"""

from .query import Query
from .query_constraints import QueryConstraints
from .query_hint_key import QueryHintKey
from .vector import VectorQueryBuilder
from .vector import FilterFactory



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from pygw.config import geowave_pkg
from pygw.store import DataStoreOptions

class RocksDBOptions(DataStoreOptions):
    """
    RocksDB data store options.
    """

    def __init__(self):
        """
        Initializes the RocksDB options class.
        """
        super().__init__(geowave_pkg.datastore.rocksdb.config.RocksDBOptions())

    def set_directory(self, directory):
        """
        Sets the directory of the RocksDB database.

        Args:
            directory (str): The directory for the database.
        """
        self._java_ref.setDirectory(directory)

    def get_directory(self):
        """
        Returns:
            The directory for the RocksDB database.
        """
        return self._java_ref.getDirectory()

    def set_compact_on_write(self, compact_on_write):
        """
        Sets whether or not to perform compaction on write.

        Args:
            compact_on_write (bool): Whether or not to perform compaction on write.
        """
        self._java_ref.setCompactOnWrite(compact_on_write)

    def is_compact_on_write(self):
        """
        Returns:
            True if compaction on write is enabled, False otherwise.
        """
        return self._java_ref.isCompactOnWrite()

    def set_batch_write_size(self, batch_write_size):
        """
        Sets the number of entries to be gathered before performing a batch write
        operation on the data store.

        Args:
            batch_write_size (int): The number of entries to write in batch write operations.
        """
        self._java_ref.setBatchWriteSize(batch_write_size)

    def get_batch_write_size(self):
        """
        Returns:
            The number of entries to write in batch write operations.
        """
        return self._java_ref.getBatchWriteSize()



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from pygw.base import GeoWaveObject

class DataStoreOptions(GeoWaveObject):
    """
    Base options for all data store types.
    """

    def __init__(self, java_ref):
        super().__init__(java_ref)
        self._base_options = java_ref.getStoreOptions()

    def set_geowave_namespace(self, namespace):
        """
        Sets the GeoWave namespace for the data store.

        Args:
            namespace (str): The namespace to use.
        """
        self._java_ref.setGeoWaveNamespace(namespace)

    def get_geowave_namespace(self):
        """
        Returns:
            The GeoWave namespace for the data store.
        """
        return self._java_ref.getGeoWaveNamespace()

    def set_persist_data_statistics(self, persist_data_statistics):
        """
        Sets whether or not to persist data statistics.

        Args:
            persist_data_statistics (bool): Whether or not to persist data statistics.
        """
        self._base_options.setPersistDataStatistics(persist_data_statistics)

    def is_persist_data_statistics(self):
        """
        Returns:
            True if data statistics are persisted, False otherwise.
        """
        return self._base_options.isPersistDataStatistics()

    def set_secondary_indexing(self, secondary_indexing):
        """
        Sets whether or not to enable secondary indexing on the data store.

        Args:
            secondary_indexing (bool): Whether or not to enable secondary indexing.
        """
        self._base_options.setSecondaryIndexing(secondary_indexing)

    def is_secondary_indexing(self):
        """
        Returns:
            True if secondary indexing is enabled, False otherwise.
        """
        return self._base_options.isSecondaryIndexing()

    def set_enable_block_cache(self, enable_block_cache):
        """
        Sets whether or not to enable the block cache.

        Args:
            enable_block_cache (bool): Whether or not to enable block cache.
        """
        self._base_options.setEnableBlockCache(enable_block_cache)

    def is_enable_block_cache(self):
        """
        Returns:
            True if block cache is enabled, False otherwise.
        """
        return self._base_options.isEnableBlockCache()

    def set_server_side_library_enabled(self, enable_server_side_library):
        """
        Sets whether or not to enable server side processing when it is available.

        Args:
            enable_server_side_library (bool): Whether or not to enable server side processing.
        """
        self._base_options.setServerSideLibraryEnabled(enable_server_side_library)

    def is_server_side_library_enabled(self):
        """
        Returns:
            True if server side processing is enabled, False otherwise.
        """
        return self._base_options.isServerSideLibraryEnabled()

    def set_max_range_decomposition(self, max_range_decomposition):
        """
        Sets the maximum number of ranges that a query can be decomposed into.

        Args:
            max_range_decomposition (int): The maximum range decomposition.
        """
        self._base_options.setMaxRangeDecomposition(max_range_decomposition)

    def get_max_range_decomposition(self):
        """
        Returns:
            The maximum range decomposition for queries.
        """
        return self._base_options.getMaxRangeDecomposition()

    def set_aggregation_max_range_decomposition(self, max_range_decomposition):
        """
        Sets the maximum number of ranges that an aggregation query can be decomposed into.

        Args:
            max_range_decomposition (int): The maximum range decomposition.
        """
        self._base_options.setAggregationMaxRangeDecomposition(max_range_decomposition)

    def get_aggregation_max_range_decomposition(self):
        """
        Returns:
            The maximum range decomposition for aggregations.
        """
        return self._base_options.getAggregationMaxRangeDecomposition()

    def set_enable_visibility(self, enable_visibility):
        """
        Sets whether or not to enable visibility on the data store.

        Args:
            enable_visibility (bool): Whether or not to enable visibility.
        """
        self._base_options.setEnableVisibility(enable_visibility)

    def is_visibility_enabled(self):
        """
        Returns:
            True if visibility is enabled, False otherwise.
        """
        return self._base_options.isVisibilityEnabled()



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================
"""
The `config` module includes a singleton object of type GeoWaveConfiguration called `gw_config` that handles all communication between python and the Py4J Java Gateway.
The module includes several shortcut objects to make accessing the gateway more convenient.  These include:
- *`java_gateway`* Py4J Gateway Object
- *`java_pkg`*: Shortcut for `java_gateway.jvm`.  Can be used to construct JVM objects like `java_pkg.org.geotools.feature.simple.SimpleFeatureTypeBuilder()`
- *`geowave_pkg`*: Similar to `java_pkg`, serves as a shortcut for `java_gateway.jvm.org.locationtech.geowave`.
- *`reflection_util`*: Direct access to the Py4J reflection utility.

These objects can be imported directly using `from pygw.config import <object_name>`.

By default, the configuration will attempt to connect to a Py4J Java Gateway running on the local machine.  To use a gateway on a separate machine, the address and port
can be specified with the `PYGW_GATEWAY_ADDRESS` and `PYGW_GATEWAY_PORT` environment variables.  Alternatively, a JavaGateway can be supplied by creating a `pygw_gateway_config`
module on the Python path that contains a JavaGateway named `gateway`.

NOTE: the GeoWaveConfiguration has an `init()` method. This is INTENTIONALLY not an `__init__` method. Initialization is attempted when the configuration is imported.
"""
from os import environ

from py4j.java_gateway import JavaGateway
from py4j.java_gateway import GatewayParameters
from py4j.java_gateway import java_import
from py4j.java_gateway import DEFAULT_ADDRESS
from py4j.java_gateway import DEFAULT_PORT

from py4j.protocol import Py4JNetworkError

PYGW_ADDRESS_ENV = 'PYGW_GATEWAY_ADDRESS'
PYGW_PORT_ENV = 'PYGW_GATEWAY_PORT'

class GeoWaveConfiguration:
    """
    This class sets up communication between Python and the GeoWave logic running
    on a JVM.
    """

    def __new__(cls):
        if not hasattr(cls, 'instance'):
            cls.instance = super(GeoWaveConfiguration, cls).__new__(cls)
        return cls.instance

    def __init__(self):
        self.is_initialized = False

    def init(self):
        """
        Sets up the Py4J Gateway, called automatically on import.
        """
        if not self.is_initialized:
            try:
                try:
                    from pygw_gateway_config import gateway
                    self.GATEWAY = gateway
                except ImportError:
                    gateway_address = environ.get(PYGW_ADDRESS_ENV, DEFAULT_ADDRESS)
                    gateway_port = environ.get(PYGW_PORT_ENV, DEFAULT_PORT)
                    self.GATEWAY = JavaGateway(gateway_parameters=GatewayParameters(auto_field=True, address=gateway_address, port=gateway_port))

                self.PKG = self.GATEWAY.jvm
                self.GEOWAVE_PKG = self.GATEWAY.jvm.org.locationtech.geowave

                ### Reflection utility ###
                self.reflection_util= self.GATEWAY.jvm.py4j.reflection.ReflectionUtil
                self.is_initialized = True
            except Py4JNetworkError as e:
                raise GeoWaveConfiguration.PyGwJavaGatewayNotStartedError("The GeoWave Py4J Java Gateway must be running before you can use pygw.") from e

    class PyGwJavaGatewayNotStartedError(Exception): pass

gw_config = GeoWaveConfiguration()
gw_config.init()

java_gateway = gw_config.GATEWAY
"""py4j.java_gateway.JavaGateway: The gateway between pygw and the JVM."""

java_pkg = gw_config.PKG
"""py4j.java_gateway.JVMView: A shortcut for accessing java packages directly.

For example `java_pkg.org.geotools.feature.simple.SimpleFeatureTypeBuilder`.
"""

geowave_pkg = gw_config.GEOWAVE_PKG
"""py4j.java_gateway.JVMView: A shortcut for accessing geowave packages directly.

For example `geowave_pkg.core.store.api.DataStoreFactory`.
"""

reflection_util = gw_config.reflection_util
"""py4j.java_gateway.JavaClass: A Java reflection utility."""

__all__ = ["java_gateway", "java_pkg", "geowave_pkg", "reflection_util"]



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from pygw.config import geowave_pkg

from .index_builder import IndexBuilder
from .index import Index

class SpatialTemporalIndexBuilder(IndexBuilder):
    """
    Index builder for a spatial temporal index.
    """
    
    def __init__(self):
        super().__init__(geowave_pkg.core.geotime.index.api.SpatialTemporalIndexBuilder())



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from pygw.base import GeoWaveObject
from pygw.base.type_conversions import GeometryType
from pygw.base.type_conversions import DateType
from pygw.config import geowave_pkg

from ..query_constraints import QueryConstraints

class SpatialTemporalConstraintsBuilder(GeoWaveObject):
    """
    A class that wraps the functionality of the GeoWave SpatialTemporalConstraintsBuilder.
    This class should not be constructed directly and instead should be created using
    the vector query constraints factory.
    """

    def no_spatial_constraints(self):
        """
        Use no spatial constraints in the query.  This is the default.

        Returns:
            This spatial temporal constraints builder.
        """
        self._java_ref.noSpatialConstraints()
        return self

    def spatial_constraints(self, geometry):
        """
        Use the given geometry as a spatial constraint.

        Args:
            geometry (shapely.geometry.base.BaseGeometry): The geometry to use in
                the constraint.
        Returns:
            This spatial temporal constraints builder.
        """
        self._java_ref.spatialConstraints(GeometryType().to_java(geometry))
        return self

    def spatial_constraints_crs(self, crs_code):
        """
        Speciify the coordinate reference system to use for the constraint.

        Args:
            crs_code (str): The CRS code to use.
        Returns:
            This spatial temporal constraints builder.
        """
        self._java_ref.spatialConstraintsCrs(crs_code)
        return self

    def spatial_constraints_compare_operation(self, spatial_compare_op):
        """
        Specify the spatial compare operation to use in conjuction with the provided
        spatial constraint.  Default is `intersects`. Possible values are `contains`,
        `overlaps`, `intersects`, `touches`, `within`, `disjoint`, `crosses`, `equals`.

        Args:
            spatial_compare_op (str): The spatial compare operation to use.
        Returns:
            This spatial temporal constraints builder.
        """
        j_compare_op = geowave_pkg.core.geotime.store.query.filter.SpatialQueryFilter.CompareOperation.valueOf(spatial_compare_op.upper())
        self._java_ref.spatialConstraintsCompareOperation(j_compare_op)
        return self

    def no_temporal_constraints(self):
        """
        Use no temporal constraints with the query.  This is the default.

        Returns:
            This spatial temporal constraints builder.
        """
        self.java_ref.noTemporalConstraints()
        return self

    def add_time_range(self, start_time, end_time):
        """
        Add a time range constraint to the query.

        Args:
            start_time (datetime): The start time of the constraint (inclusive).
            end_time (datetime): The end time of the constraint (exclusive).
        Returns:
            This spatial temporal constraints builder.
        """
        dt = DateType()
        self._java_ref.addTimeRange(dt.to_java(start_time), dt.to_java(end_time))
        return self

    def build(self):
        """
        Builds the configured spatial temporal constraint.

        Returns:
            A `pygw.query.query_constraints.QueryConstraints` with the configured spatial/temporal constraints.
        """
        return QueryConstraints(self._java_ref.build())



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================
"""
This module contains classes that wrap the functionality of geotools SimpleFeatures and SimpleFeatureTypes.  These classes can be used to create feature types, features, and data adapters based on simple features.

It contains the following import shortcuts:
```python
from pygw.geotools import FeatureDataAdapter
from pygw.geotools import AttributeDescriptor
from pygw.geotools import SimpleFeature
from pygw.geotools import SimpleFeatureBuilder
from pygw.geotools import SimpleFeatureType
from pygw.geotools import SimpleFeatureTypeBuilder
```
"""

from .feature_data_adapter import FeatureDataAdapter
from .attribute_descriptor import AttributeDescriptor
from .simple_feature_type import SimpleFeatureType
from .simple_feature_type_builder import SimpleFeatureTypeBuilder
from .simple_feature import SimpleFeature
from .simple_feature_builder import SimpleFeatureBuilder



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

import pytest
import os
from datetime import datetime

from shapely.geometry import Polygon

from pygw.store import DataStoreFactory
from pygw.store.rocksdb import RocksDBOptions
from pygw.index import SpatialIndexBuilder
from pygw.index import SpatialTemporalIndexBuilder
from pygw.query import VectorQueryBuilder
from pygw.query import FilterFactory

from .conftest import TEST_DIR
from .conftest import POINT_TYPE_ADAPTER
from .conftest import TEST_DATA
from .conftest import write_test_data
from .conftest import results_as_list

# Test Deleting #
def test_cql_query(test_ds):
    # given
    index = SpatialIndexBuilder().set_name("idx1").create_index()
    adapter = POINT_TYPE_ADAPTER
    test_ds.add_type(adapter, index)
    write_test_data(test_ds, index)

    # when
    qbldr = VectorQueryBuilder()
    constraints_factory = qbldr.constraints_factory()
    # filter encompasses 10 features (1, 1) - (10, 10)
    constraints = constraints_factory.cql_constraints("BBOX(the_geom, 0.5, 0.5, 10.5, 10.5)")
    qbldr.constraints(constraints)
    res = results_as_list(test_ds.query(qbldr.build()))

    # then
    assert len(res) == 10

def test_query_spatial(test_ds):
    # given
    index = SpatialIndexBuilder().set_name("idx1").create_index()
    adapter = POINT_TYPE_ADAPTER
    test_ds.add_type(adapter, index)
    write_test_data(test_ds, index)

    # when
    qbldr = VectorQueryBuilder()
    stcb = qbldr.constraints_factory().spatial_temporal_constraints()
    # polygon encompasses 10 features (1, 1) - (10, 10)
    stcb.spatial_constraints(Polygon([[0.5, 0.5], [0.5, 10.5], [10.5, 10.5], [10.5, 0.5], [0.5, 0.5]]))
    stcb.spatial_constraints_compare_operation("CONTAINS")
    qbldr.constraints(stcb.build())
    res = results_as_list(test_ds.query(qbldr.build()))

    # then
    assert len(res) == 10

def test_query_temporal(test_ds):
    # given
    index = SpatialIndexBuilder().set_name("idx1").create_index()
    adapter = POINT_TYPE_ADAPTER
    test_ds.add_type(adapter, index)
    write_test_data(test_ds, index)

    # when
    qbldr = VectorQueryBuilder()
    stcb = qbldr.constraints_factory().spatial_temporal_constraints()
    # time range encompasses 10 features (1, 1) - (10, 10)
    stcb.add_time_range(datetime.fromtimestamp(1), datetime.fromtimestamp(11))
    qbldr.constraints(stcb.build())
    res = results_as_list(test_ds.query(qbldr.build()))

    for feature in res:
        print(feature.get_id())
        print(feature.get_default_geometry())
    # then
    assert len(res) == 10

def test_query_spatial_temporal(test_ds):
    # given
    index = SpatialIndexBuilder().set_name("idx1").create_index()
    adapter = POINT_TYPE_ADAPTER
    test_ds.add_type(adapter, index)
    write_test_data(test_ds, index)

    # when
    qbldr = VectorQueryBuilder()
    stcb = qbldr.constraints_factory().spatial_temporal_constraints()
    # polygon encompasses 10 features (1, 1) - (10, 10)
    stcb.spatial_constraints(Polygon([[0.5, 0.5], [0.5, 10.5], [10.5, 10.5], [10.5, 0.5], [0.5, 0.5]]))
    stcb.spatial_constraints_compare_operation("CONTAINS")
    # time range encompasses 10 features (5, 5) - (14, 14)
    stcb.add_time_range(datetime.fromtimestamp(5), datetime.fromtimestamp(15))
    qbldr.constraints(stcb.build())
    res = results_as_list(test_ds.query(qbldr.build()))

    # then
    assert len(res) == 6

def test_query_filter(test_ds):
    # given
    index = SpatialIndexBuilder().set_name("idx1").create_index()
    adapter = POINT_TYPE_ADAPTER
    test_ds.add_type(adapter, index)
    write_test_data(test_ds, index)

    # when
    qbldr = VectorQueryBuilder()
    filter_factory = FilterFactory()
    # filter encompasses 10 features (1, 1) - (10, 10)
    bbox_filter = filter_factory.bbox(filter_factory.property("the_geom"), 0.5, 0.5, 10.5, 10.5, "EPSG:4326")
    constraints = qbldr.constraints_factory().filter_constraints(bbox_filter)
    qbldr.constraints(constraints)
    res = results_as_list(test_ds.query(qbldr.build()))

    # then
    assert len(res) == 10



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from pygw.config import geowave_pkg
from pygw.store import DataStoreOptions

class CassandraOptions(DataStoreOptions):
    """
    Cassandra data store options.
    """

    def __init__(self):
        super().__init__(geowave_pkg.datastore.cassandra.config.CassandraRequiredOptions())

    def set_contact_point(self, contact_point):
        """
        Sets a single contact point or a comma delimited set of contact points to
        connect to the Cassandra cluster.

        Args:
            contact_point (str): The contact point(s) to connect to.
        """
        self._java_ref.setContactPoint(contact_point)

    def get_contact_point(self):
        """
        Returns:
            The contact points of the Cassandra cluster.
        """
        return self._java_ref.getContactPoint()

    def set_batch_write_size(self, batch_write_size):
        """
        Sets the number of entries to be gathered before performing a batch write
        operation on the data store.

        Args:
            batch_write_size (int): The number of entries to write in batch write operations.
        """
        self._base_options.setBatchWriteSize(batch_write_size)

    def get_batch_write_size(self):
        """
        Returns:
            The number of entries to write in batch write operations.
        """
        return self._base_options.getBatchWriteSize()

    def set_durable_writes(self, durable_writes):
        """
        Sets whether or not to write to commit log for durability, configured only
        on creation of new keyspace.

        Args:
            durable_writes (bool): Whether or not to enable durable writes.
        """
        self._base_options.setDurableWrites(durable_writes)

    def is_durable_writes(self):
        """
        Returns:
            True if durable writes are enabled, False otherwise.
        """
        return self._base_options.isDurableWrites()

    def set_replication_factor(self, replication_factor):
        """
        Sets the number of replicas to use when creating a new keyspace.

        Args:
            replication_factor (int): The number of replicas.
        """
        self._base_options.setReplicationFactor(replication_factor)

    def get_replication_factor(self):
        """
        Returns:
            The number of replicas to use when creating a new keyspace.
        """
        return self._base_options.getReplicationFactor()



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from pygw.base import GeoWaveObject
from pygw.base.type_conversions import StringArrayType

from .query import Query
from .query_hint_key import QueryHintKey
from .query_constraints import QueryConstraints

class QueryBuilder(GeoWaveObject):
    """
    Base query builder for constructing GeoWave queries.  This class should not
    be used directly.  Instead, use one of the derived classes such as
    `pygw.query.vector.vector_query_builder.VectorQueryBuilder`.
    """

    def __init__(self, java_ref, result_transformer):
        self._result_transformer = result_transformer
        super().__init__(java_ref)

    def constraints_factory():
        """
        Creates a constraints factory for the type of query that is being built.
        This should be overridden by all derived query builders.

        Raises:
            NotImplementedError: This should be overridden by derived query builders.
        Returns:
            An appropriate `pygw.query.query_constraints_factory.QueryConstraintsFactory` for the query type.
        """
        raise NotImplementedError

    def all_types(self):
        """
        Configure the query to get data from all types. This is the default.

        Returns:
            This query builder.
        """
        self._java_ref.allTypes()
        return self

    def add_type_name(self, type_name):
        """
        Configure the query to get data from a specific type.

        Args:
            type_name (str): The type to get data from.
        Returns:
            This query builder.
        """
        self._java_ref.addTypeName(type_name)
        return self

    def set_type_names(self, type_names):
        """
        Configure the query to get data from a set of types.

        Args:
            type_names (list of str): The types to get data from.
        Returns:
            This query builder.
        """
        self._java_ref.setTypeNames(StringArrayType().to_java(type_names))
        return self

    def subset_fields(self, type_name, field_names):
        """
        Configure the query to get a specific set of fields from a given type.

        Args:
            type_name (str): The type to get from.
            field_names (list of str): The fields to get.
        Returns:
            This query builder.
        """
        self._java_ref.subsetFields(type_name, StringArrayType().to_java(field_names))
        return self

    def all_fields(self):
        """
        Configure the query to get all fields from the given type(s). This is the
        default.

        Returns:
            This query builder.
        """
        self._java_ref.allFields()
        return self

    def all_indices(self):
        """
        Configure the query to allow the use of all indices when getting data.
        This is the default.

        Returns:
            This query builder.
        """
        self._java_ref.allIndices()
        return self

    def index_name(self, index_name):
        """
        Configure the query to get data from a specific index.

        Args:
            index_name (str): The name of the index to get data from.
        Returns:
            This query builder.
        """
        self._java_ref.indexName(index_name)
        return self

    def add_authorization(self, authorization):
        """
        Configure the query to get data using the given authorization.

        Args:
            authorization (str): The authorization to use in the query.
        Returns:
            This query builder.
        """
        self._java_ref.addAuthorization(authorization)
        return self

    def set_authorizations(self, authorizations):
        """
        Configure the query to get data using the given set of authorizations.

        Args:
            authorizations (list of str): The authorizations to use in the query.
        Returns:
            This query builder.
        """
        self._java_ref.setAuthorizations(StringArrayType().to_java(authorizations))
        return self

    def no_authorizations(self):
        """
        Configure the query to get data without using any authorizations.  This
        is the default.

        Returns:
            This query builder.
        """
        self._java_ref.noAuthorizations()
        return self

    def no_limit(self):
        """
        Configure the query to get all results that match the query constraints.
        This is the default.

        Returns:
            This query builder.
        """
        self._java_ref.noLimit()
        return self

    def limit(self, limit):
        """
        Configure the query to only return a limited number of results.

        Args:
            limit (int): The maximum number of results to get.
        Returns:
            This query builder.
        """
        self._java_ref.limit(limit)
        return self

    def add_hint(self, key, value):
        """
        Adds a hint to the query.  Available query hints are defined by the
        enumeration at `pygw.query.query_hint_key.QueryHintKey`.

        Args:
            key (pygw.query.query_hint_key.QueryHintKey): The key of the hint to set.
            value (any): The value to use for the hint.
        Returns:
            This query builder.
        """
        assert isinstance(key, QueryHintKey)
        self._java_ref.addHint(QueryHintKey.get_key(key), value)
        return self

    def no_hints(self):
        """
        Configure the query to use no query hints.  This is the default.

        Returns:
            This query builder.
        """
        self._java_ref.noHints()
        return self

    def constraints(self, constraints):
        """
        Configure the query to be constrained by the given query constraints.
        Constraints can be constructed using the factory provided by the
        `pygw.query.query_builder.QueryBuilder.constraints_factory` method.

        Args:
            constraints (pygw.query.query_constraints.QueryConstraints): The constraints to use.
        Returns:
            This query builder.
        """
        assert isinstance(constraints, QueryConstraints)
        self._java_ref.constraints(constraints._java_ref)
        return self

    def build(self):
        """
        Builds the configured query.

        Returns:
            The final constructed `pygw.query.query.Query`.
        """
        return Query(self._java_ref.build(), self._result_transformer)



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from pygw.base.type_conversions import *
from pygw.geotools import SimpleFeatureTypeBuilder
from pygw.geotools import SimpleFeatureBuilder
from pygw.geotools import AttributeDescriptor

def check_attribute(sft, attr_name, nilable, type):
    attr = sft.get_attribute(attr_name)
    assert attr.descriptor == attr_name
    assert attr.is_nilable == nilable
    assert isinstance(attr.field, type)

def test_simple_feature_type():
    sftb = SimpleFeatureTypeBuilder()
    sftb.set_name("TestKitchenSinkType")
    sftb.set_namespace_uri("http://www.example.org")
    sftb.set_srs("EPSG:4326")
    sftb.add(AttributeDescriptor.point("the_geom", False))
    sftb.add(AttributeDescriptor.big_decimal("big_decimal", True))
    sftb.add(AttributeDescriptor.big_decimal_array("big_decimal_array"))
    sftb.add(AttributeDescriptor.big_integer("big_integer"))
    sftb.add(AttributeDescriptor.big_integer_array("big_integer_array"))
    sftb.add(AttributeDescriptor.boolean("boolean"))
    sftb.add(AttributeDescriptor.boolean_array("boolean_array"))
    sftb.add(AttributeDescriptor.primitive_boolean_array("prim_boolean_array"))
    sftb.add(AttributeDescriptor.float("float"))
    sftb.add(AttributeDescriptor.float_array("float_array"))
    sftb.add(AttributeDescriptor.primitive_float_array("prim_float_array"))
    sftb.add(AttributeDescriptor.double("double"))
    sftb.add(AttributeDescriptor.double_array("double_array"))
    sftb.add(AttributeDescriptor.primitive_double_array("prim_double_array"))
    sftb.add(AttributeDescriptor.byte("byte"))
    sftb.add(AttributeDescriptor.byte_array("byte_array"))
    sftb.add(AttributeDescriptor.primitive_byte_array("prim_byte_array"))
    sftb.add(AttributeDescriptor.short("short"))
    sftb.add(AttributeDescriptor.short_array("short_array"))
    sftb.add(AttributeDescriptor.primitive_short_array("prim_short_array"))
    sftb.add(AttributeDescriptor.integer("integer"))
    sftb.add(AttributeDescriptor.integer_array("integer_array"))
    sftb.add(AttributeDescriptor.primitive_int_array("prim_int_array"))
    sftb.add(AttributeDescriptor.long("long"))
    sftb.add(AttributeDescriptor.long_array("long_array"))
    sftb.add(AttributeDescriptor.primitive_long_array("prim_long_array"))
    sftb.add(AttributeDescriptor.string("string"))
    sftb.add(AttributeDescriptor.string_array("string_array"))
    sftb.add(AttributeDescriptor.date("date"))
    sftb.add(AttributeDescriptor.date_array("date_array"))
    sftb.add(AttributeDescriptor.calendar("calendar"))
    sftb.add(AttributeDescriptor.calendar_array("calendar_array"))
    sftb.add(AttributeDescriptor.point_array("point_array"))
    sftb.add(AttributeDescriptor.multi_point("multi_point"))
    sftb.add(AttributeDescriptor.multi_point_array("multi_point_array"))
    sftb.add(AttributeDescriptor.line_string("line_string"))
    sftb.add(AttributeDescriptor.line_string_array("line_string_array"))
    sftb.add(AttributeDescriptor.multi_line_string("multi_line_string"))
    sftb.add(AttributeDescriptor.multi_line_string_array("multi_line_string_array"))
    sftb.add(AttributeDescriptor.polygon("polygon"))
    sftb.add(AttributeDescriptor.polygon_array("polygon_array"))
    sftb.add(AttributeDescriptor.multi_polygon("multi_polygon"))
    sftb.add(AttributeDescriptor.multi_polygon_array("multi_polygon_array"))
    sftb.add(AttributeDescriptor.geometry_collection("geometry_collection"))
    sftb.add(AttributeDescriptor.geometry_collection_array("geometry_collection_array"))
    sftb.add(AttributeDescriptor.geometry("geometry"))
    sftb.add(AttributeDescriptor.geometry_array("geometry_array"))
    sft = sftb.build_feature_type()

    assert sft.get_type_name() == "TestKitchenSinkType"
    check_attribute(sft, "the_geom", False, PointType)
    check_attribute(sft, "big_decimal", True, BigDecimalType)
    check_attribute(sft, "big_decimal_array", False, BigDecimalArrayType)
    check_attribute(sft, "big_integer", False, BigIntegerType)
    check_attribute(sft, "big_integer_array", False, BigIntegerArrayType)
    check_attribute(sft, "boolean", False, BooleanType)
    check_attribute(sft, "boolean_array", False, BooleanArrayType)
    check_attribute(sft, "prim_boolean_array", False, PrimitiveBooleanArrayType)
    check_attribute(sft, "float", False, FloatType)
    check_attribute(sft, "float_array", False, FloatArrayType)
    check_attribute(sft, "prim_float_array", False, PrimitiveFloatArrayType)
    check_attribute(sft, "double", False, DoubleType)
    check_attribute(sft, "double_array", False, DoubleArrayType)
    check_attribute(sft, "prim_double_array", False, PrimitiveDoubleArrayType)
    check_attribute(sft, "byte", False, ByteType)
    check_attribute(sft, "byte_array", False, ByteArrayType)
    check_attribute(sft, "prim_byte_array", False, PrimitiveByteArrayType)
    check_attribute(sft, "short", False, ShortType)
    check_attribute(sft, "short_array", False, ShortArrayType)
    check_attribute(sft, "prim_short_array", False, PrimitiveShortArrayType)
    check_attribute(sft, "integer", False, IntegerType)
    check_attribute(sft, "integer_array", False, IntegerArrayType)
    check_attribute(sft, "prim_int_array", False, PrimitiveIntArrayType)
    check_attribute(sft, "long", False, LongType)
    check_attribute(sft, "long_array", False, LongArrayType)
    check_attribute(sft, "prim_long_array", False, PrimitiveLongArrayType)
    check_attribute(sft, "string", False, StringType)
    check_attribute(sft, "string_array", False, StringArrayType)
    check_attribute(sft, "date", False, DateType)
    check_attribute(sft, "date_array", False, DateArrayType)
    check_attribute(sft, "calendar", False, CalendarType)
    check_attribute(sft, "calendar_array", False, CalendarArrayType)
    check_attribute(sft, "point_array", False, PointArrayType)
    check_attribute(sft, "multi_point", False, MultiPointType)
    check_attribute(sft, "multi_point_array", False, MultiPointArrayType)
    check_attribute(sft, "line_string", False, LineStringType)
    check_attribute(sft, "line_string_array", False, LineStringArrayType)
    check_attribute(sft, "multi_line_string", False, MultiLineStringType)
    check_attribute(sft, "multi_line_string_array", False, MultiLineStringArrayType)
    check_attribute(sft, "polygon", False, PolygonType)
    check_attribute(sft, "polygon_array", False, PolygonArrayType)
    check_attribute(sft, "multi_polygon", False, MultiPolygonType)
    check_attribute(sft, "multi_polygon_array", False, MultiPolygonArrayType)
    check_attribute(sft, "geometry_collection", False, GeometryCollectionType)
    check_attribute(sft, "geometry_collection_array", False, GeometryCollectionArrayType)
    check_attribute(sft, "geometry", False, GeometryType)
    check_attribute(sft, "geometry_array", False, GeometryArrayType)

    # Get Attribute by index
    assert sft.get_attribute(2).descriptor == "big_decimal_array"
    assert sft.get_attribute(15).descriptor == "byte_array"

    # Get non-existent attribute
    assert sft.get_attribute("nonexistent") is None

def test_simple_feature():
    sftb = SimpleFeatureTypeBuilder()
    sftb.set_name("TestPointType")
    sftb.add(AttributeDescriptor.point("the_geom", False))
    sftb.add(AttributeDescriptor.big_integer("big_int", True))
    sftb.add(AttributeDescriptor.string("string", True))
    sftb.add(AttributeDescriptor.byte("byte", False))
    sftb.add(AttributeDescriptor.float("float", True))
    sft = sftb.build_feature_type()

    big_number = 123123123123123123123123123
    sfb = SimpleFeatureBuilder(sft)
    sfb.set_attr("the_geom", Point(1, 1))
    sfb.set_attr("big_int", big_number)
    sfb.set_attr("string", "test value")
    sfb.set_attr("byte", 38)
    feature = sfb.build("fid1")

    assert feature.get_id() == "fid1"
    assert feature.get_type() is sft
    assert feature.get_feature_type() is sft
    assert feature.get_default_geometry() == Point(1, 1)
    assert feature.get_attribute_count() == 5
    assert feature.get_attribute("string") == "test value"
    assert feature.get_attribute(1) == big_number
    attrs = feature.get_attributes()
    assert attrs[0] == Point(1, 1)
    assert attrs[1] == big_number
    assert attrs[2] == "test value"
    assert attrs[3] == 38
    assert attrs[4] is None

    feature_dict = feature.to_dict()
    assert feature_dict["id"] == "fid1"
    assert feature_dict["the_geom"] == Point(1, 1)
    assert feature_dict["string"] == "test value"
    assert feature_dict["byte"] == 38
    assert feature_dict["big_int"] == big_number



#
# Copyright (c) 2013-2020 Contributors to the Eclipse Foundation

#
# See the NOTICE file distributed with this work for additional information regarding copyright
# ownership. All rights reserved. This program and the accompanying materials are made available
# under the terms of the Apache License, Version 2.0 which accompanies this distribution and is
# available at http://www.apache.org/licenses/LICENSE-2.0.txt
#===============================================================================================

from pygw.config import java_gateway

class GeoWaveObject:
    """
    Base Class for pygw objects that wrap Java objects.
    """

    def __init__(self, java_ref):
        self._java_ref = java_ref

    def __repr__(self):
        return "pygw {} Object with JavaRef@{}".format(self.__class__, self._java_ref)

    def __eq__(self, other):
        if not isinstance(other, PyGwJavaWrapper):
            return False
        return self._java_ref == other._java_ref

    def is_instance_of(self, java_class):
        """
        Returns:
            True if this object is of the type represented by the given java class.
        """
        return isinstance(java_gateway, self._java_ref, java_class)
